%{
    #include<iostream>
    #include<string>
    #include<vector>
    #include<map>
#ifdef TARGET_PLATFORM_WASM
    #include <sstream>
    #include <stdio.h> // for fmemopen
    #include <emscripten.h>
#endif

    using namespace std;
    #define YYERROR_VERBOSE


    bool DEBUG_LEXER = false;
    bool DEBUG_PARSER = false;
    bool verbose = false;
    bool silent = false;

    
    class MuRecursiveFunction
    {
        public:
            size_t arity;
            
        public:
            MuRecursiveFunction(size_t arity)
            {
                this->arity = arity;
            }
            
            virtual ~MuRecursiveFunction()
            {
            }

            
            virtual bool ValidStructure() const
            {
                return true;
            }
            
            virtual bool AllVariablesDefined(const vector<string>& names) const
            {
                return true;
            }
            
            virtual MuRecursiveFunction* Clone() const = 0;
           
            virtual bool CanApplyMacro(const map<string, MuRecursiveFunction*>&  replacements, string*) const
            {
                return true;
            }
            
            virtual MuRecursiveFunction* ApplyMacro(const map<string, MuRecursiveFunction*>&  replacements)
            {
                return this;
            }

            virtual bool ContainsVariables() const
            {
                return false;
            }
            
            // if it returns false, then the parameter must point to a string explaining the reason of failure
            virtual bool CanComputeArity(string*) const
            {
                return true;
            }

            virtual size_t ComputeArity() const
            {
                return arity;
            }
            
            bool CanApply(const vector<unsigned int>& Parameters, string* reason) const
            {
                if(!ValidStructure())
                {
                    *reason = "invalid function";
                    return false;
                }
                
                if(ContainsVariables())
                {
                    *reason = "contains undefined variables";
                    return false;
                }
                
                if(!CanComputeArity(reason))
                    return false;
                    
                size_t ar = ComputeArity();
                if(ar != Parameters.size())
                {
                    *reason = "wrong number of parameters";
                    return false;
                }
                
                return true;
            }
            
            virtual unsigned int Apply(const vector<unsigned int>& Parameters) = 0;

            virtual bool DependsOnParameter(size_t paramPosition) const
            {
                return true;
            }
            
            virtual bool CanIterate() const
            {
                return false;
            }
            
            virtual unsigned int ApplyIterated(int n, const vector<unsigned int>& Parameters)
            {
                throw "Trying immediate iteration on object that cannot do immediate iteration";
            }
    };

    class MuRecursiveFunctionMacro
    {
        public:
            vector<string>* params;
            MuRecursiveFunction* function;
        public:
            MuRecursiveFunctionMacro(vector<string>* params, MuRecursiveFunction* function)
            {
                this->params = params;
                this->function = function;
            }
            
            ~MuRecursiveFunctionMacro()
            {
                delete params;
                delete function;
            }

            bool CanInstantiate(const vector<MuRecursiveFunction*>* parameterInstantiations, string* reason)
            {
                if(function==NULL || !function->ValidStructure())
                {
                    *reason = "trying to instantiate invalid function";
                    return false;
                }
                
                if(parameterInstantiations != NULL)
                    for(vector<MuRecursiveFunction*>::const_iterator i = parameterInstantiations->begin(); i != parameterInstantiations->end(); i++)
                        if(*i == NULL || !(*i)->ValidStructure())
                        {
                            *reason = "invalid parameter";
                            return false;
                        }
            
                if((params != NULL && parameterInstantiations != NULL && params->size() != parameterInstantiations->size())
                || (params == NULL && parameterInstantiations != NULL && parameterInstantiations->size() != 0)
                || (params != NULL && parameterInstantiations == NULL && params->size() != 0))
                {
                    *reason = "Wrong number of arguments";
                    return false;
                }
                
                map<string, MuRecursiveFunction*> parametermap;
                if(params != NULL && parameterInstantiations != NULL)
                {
                    vector<MuRecursiveFunction*>::const_iterator p = parameterInstantiations->begin();
                    for(vector<string>::iterator pn = params->begin(); pn != params->end(); pn++, p++)
                        parametermap[*pn] = *p;
                }
                
                if(!function->CanApplyMacro(parametermap, reason))
                    return false;
                
                return true;
            }
            
            MuRecursiveFunction* Instantiate(const vector<MuRecursiveFunction*>* parameterInstantiations)
            {
                map<string, MuRecursiveFunction*> parametermap;
                
                if(params != NULL && parameterInstantiations != NULL)
                {
                    vector<MuRecursiveFunction*>::const_iterator p = parameterInstantiations->begin();
                    for(vector<string>::iterator pn = params->begin(); pn != params->end(); pn++, p++)
                        parametermap[*pn] = *p;
                }
                
                MuRecursiveFunction* clone = function->Clone();
                return clone->ApplyMacro(parametermap);
            }
    };
    
    
    
    map<string, MuRecursiveFunctionMacro*> macros;
    

    
    class Variable : public MuRecursiveFunction
    {
        protected:
            string name;
        public:
            Variable(string name)
                : MuRecursiveFunction(0)
            {
                this->name = name;
            }
            
            bool AllVariablesDefined(const vector<string>& names) const
            {
                for(vector<string>::const_iterator i = names.begin(); i != names.end(); i++)
                    if(name == *i)
                        return true;
                return false;
            }
            
            MuRecursiveFunction* Clone() const
            {
                return new Variable(name);
            }

            bool CanApplyMacro(const map<string, MuRecursiveFunction*>&  replacements, string* reason)
            {
                map<string, MuRecursiveFunction*>::const_iterator r = replacements.find(name);
                if(r == replacements.end())
                {
                    *reason = "Undefined Variable";
                    return false;
                }
                return true;
            }
            
            MuRecursiveFunction* ApplyMacro(const map<string, MuRecursiveFunction*>&  replacements)
            {
                map<string, MuRecursiveFunction*>::const_iterator r = replacements.find(name);
                return r->second->Clone();
            }
            
            bool ContainsVariables() const
            {
                return true;
            }
            
            bool CanComputeArity(string* reason) const
            {
                *reason = "Undefined Variable";
                return false;
            }
            
            size_t ComputeArity() const
            {
                throw "Variable::ComputeArity undefined variable";
            }

            bool CanApply(const vector<unsigned int>& Parameters, string* reason) const
            {
                *reason = "Undefined Variable";
                return false;
            }
            
            unsigned int Apply(const vector<unsigned int>& Parameters)
            {
                throw "Variable::Apply variables cannot be applied";
            }
    };
    
    
    
    class Constant : public MuRecursiveFunction
    {
        protected:
            unsigned int value;
        public:
            Constant(size_t arity, unsigned int value)
                : MuRecursiveFunction(arity)
            {
                this->value = value;
            }

            ~Constant()
            {
            }

            MuRecursiveFunction* Clone() const
            {
                return new Constant(arity, value);
            }
            
            unsigned int Apply(const vector<unsigned int>& Parameters)
            {
                if(Parameters.size() != arity)
                    throw "Constant::Apply wrong number of parameters!";
                return value;
            }
            
            bool DependsOnParameter(size_t paramPosition) const
            {
                return false;
            }
            
            bool CanIterate() const
            {
                return true;
            }
            
            unsigned int ApplyIterated(int n, const vector<unsigned int>& Parameters)
            {
                return Apply(Parameters);
            }
    };

    class Projection : public MuRecursiveFunction
    {
        public:
            unsigned int ProjectionPosition;
        public:
            Projection(size_t arity, size_t position)
                : MuRecursiveFunction(arity)
            {
                this->ProjectionPosition = position;
            }

            ~Projection()
            {
            }

            MuRecursiveFunction* Clone() const
            {
                return new Projection(arity, ProjectionPosition);
            }
            
            bool CanComputeArity(string* reason) const
            {
                if(ProjectionPosition >= arity)
                {
                    *reason = "Projection: index out of bounds (index > arity)";
                    return false;
                }
                return true;
            }
            
            unsigned int Apply(const vector<unsigned int>& Parameters)
            {
                if(Parameters.size() != arity)
                    throw "Projection::Apply wrong number of parameters!";
                return Parameters[ProjectionPosition];
            }
            
            bool DependsOnParameter(size_t paramPosition) const
            {
                return ProjectionPosition == paramPosition;
            }

            bool CanIterate() const
            {
                return arity == 1;
            }
            
            unsigned int ApplyIterated(int n, const vector<unsigned int>& Parameters)
            {
                return Apply(Parameters);
            }            
    };

    
    class Successor : public MuRecursiveFunction
    {
        public:
            Successor()
                : MuRecursiveFunction(1)
            {
            }
            
            ~Successor()
            {
            }
            
            MuRecursiveFunction* Clone() const
            {
                return new Successor();
            }

            unsigned int Apply(const vector<unsigned int>& Parameters)
            {
                if(Parameters.size() != arity)
                    throw "Successor::Apply wrong number of parameters!";
                return Parameters[0]+1;
            }
            
            bool DependsOnParameter(size_t paramPosition) const
            {
                return paramPosition == 0;
            }
            
            bool CanIterate() const
            {
                return true;
            }
            
            unsigned int ApplyIterated(int n, const vector<unsigned int>& Parameters)
            {
                if(Parameters.size() != arity)
                    throw "Successor::ApplyIterated wrong number of parameters!";
                return Parameters[0]+n;
            }            
    };
    
    
    class Composition : public MuRecursiveFunction
    {
        public:
            MuRecursiveFunction* outer;
            vector<MuRecursiveFunction*>* inner;
        public:
            Composition(MuRecursiveFunction* outer, vector<MuRecursiveFunction*>* inner)
                : MuRecursiveFunction(0)
            {
                this->outer = outer;
                this->inner = inner;
            }
            
            ~Composition()
            {
                delete outer;
                if(inner != NULL)
                    for(vector<MuRecursiveFunction*>::iterator i = inner->begin(); i != inner->end(); i++)
                        delete *i;
                delete inner;
            }

            bool ValidStructure() const
            {
                if(outer == NULL || inner == NULL)
                    return false;
                for(vector<MuRecursiveFunction*>::iterator i = inner->begin(); i != inner->end(); i++)
                    if(*i == NULL || !(*i)->ValidStructure())
                        return false;
                return outer->ValidStructure();
            }
            
            bool AllVariablesDefined(const vector<string>& names) const
            {
                if(!outer->AllVariablesDefined(names))
                    return false;
                for(vector<MuRecursiveFunction*>::iterator i = inner->begin(); i != inner->end(); i++)
                    if(!(*i)->AllVariablesDefined(names))
                        return false;
                return true;
            }

            MuRecursiveFunction* Clone() const
            {
                vector<MuRecursiveFunction*>* innerclones = new vector<MuRecursiveFunction*>(inner->size());
                for(vector<MuRecursiveFunction*>::iterator i=inner->begin(), c=innerclones->begin(); i != inner->end(); i++, c++)
                    *c = (*i)->Clone();
                return new Composition(outer->Clone(), innerclones);
            }

            bool CanApplyMacro(const map<string, MuRecursiveFunction*>&  replacements, string* reason) const
            {
                if(!outer->CanApplyMacro(replacements, reason))
                    return false;
                for(vector<MuRecursiveFunction*>::iterator i = inner->begin(); i != inner->end(); i++)
                    if(!(*i)->CanApplyMacro(replacements,reason))
                        return false;
                return true;
            }            
            
            MuRecursiveFunction* ApplyMacro(const map<string, MuRecursiveFunction*>&  replacements)
            {
                MuRecursiveFunction* temp = outer->ApplyMacro(replacements);
                if(temp != outer)
                {
                    delete outer;
                    outer = temp;
                }
                
                for(vector<MuRecursiveFunction*>::iterator i = inner->begin(); i != inner->end(); i++)
                {
                    temp = (*i)->ApplyMacro(replacements);
                    if(temp != *i)
                    {
                        delete (*i);
                        (*i) = temp;
                    }
                }
                
                return this;
            }            
            
            bool ContainsVariables() const
            {
                if(outer->ContainsVariables())
                    return true;
                for(vector<MuRecursiveFunction*>::iterator i = inner->begin(); i != inner->end(); i++)
                    if((*i)->ContainsVariables())
                        return true;
                return false;
            }
            
            bool CanComputeArity(string* reason) const
            {
                if(!outer->CanComputeArity(reason))
                    return false;

                if(outer->ComputeArity() != inner->size())
                {
                    *reason = "Composition: arity of outer function must be equal to the number of inner functions";
                    return false;
                }            
            
                vector<MuRecursiveFunction*>::const_iterator i = inner->begin();
                if(i != inner->end())
                {
                    if(!(*i)->CanComputeArity(reason))
                        return false;
                    size_t commonarity = (*i)->ComputeArity();
                    
                    for(i++; i != inner->end(); i++)
                    {
                        if(!(*i)->CanComputeArity(reason))
                            return false;
                            
                        if((*i)->ComputeArity() != commonarity)
                        {
                            *reason = "Composition: all inner functions must have the same arity";
                            return false;
                        }
                    }
                }
                else
                {
                    *reason = "Composition: cannot compose 0-ary functions";
                    return false;
                }
                
                return true;
            }
            
            size_t ComputeArity() const
            {
                vector<MuRecursiveFunction*>::const_iterator i = inner->begin();
                return (*i)->ComputeArity();            
            }

            unsigned int Apply(const vector<unsigned int>& Parameters)
            {
                vector<unsigned int> temp(inner->size());
                size_t j = 0;
                for(vector<MuRecursiveFunction*>::iterator i = inner->begin(); i != inner->end(); i++, j++)
                    if(outer->DependsOnParameter(j))
                        temp[j] = (*i)->Apply(Parameters);
                    else
                        temp[j] = 0;
                return outer->Apply(temp);
            }
            
            bool DependsOnParameter(size_t paramPosition) const
            {
                size_t iindex = 0;
                for(vector<MuRecursiveFunction*>::const_iterator i = inner->begin(); i != inner->end(); i++, iindex++)
                    if(outer->DependsOnParameter(iindex) && (*i)->DependsOnParameter(paramPosition))
                        return true;
                return false;
            }

            bool CanIterate() const
            {
                if(!outer->CanIterate())
                    return false;
                MuRecursiveFunction* param1 = inner->size() == 1 ? *(inner->begin()) : NULL;
                Projection* proj = dynamic_cast<Projection*>(param1);
                if(proj == NULL)
                    return false;
                return proj->ProjectionPosition+1 == proj->arity;
            }
            
            unsigned int ApplyIterated(int n, const vector<unsigned int>& Parameters)
            {
                vector<unsigned int> phony;
                phony.push_back(Parameters[Parameters.size()-1]);
                return outer->ApplyIterated(n, phony);
            }
    };
    
    
    class PrimitiveRecursion : public MuRecursiveFunction
    {
        protected:
            MuRecursiveFunction* base;
            MuRecursiveFunction* iter;
        public:
            PrimitiveRecursion(MuRecursiveFunction* base, MuRecursiveFunction* iter)
                : MuRecursiveFunction(base->arity + 1)
            {
                this->base = base;
                this->iter = iter;
            }
            
            ~PrimitiveRecursion()
            {
                delete base;
                delete iter;
            }

            bool ValidStructure() const
            {
                if(base == NULL || iter == NULL)
                    return false;
                return base->ValidStructure() && iter->ValidStructure();
            }
            
            bool AllVariablesDefined(const vector<string>& names) const
            {
                return base->AllVariablesDefined(names)
                    && iter->AllVariablesDefined(names);
            }
            
            MuRecursiveFunction* Clone() const
            {
                return new PrimitiveRecursion(base->Clone(), iter->Clone());
            }

            bool CanApplyMacro(const map<string, MuRecursiveFunction*>&  replacements, string* reason) const
            {
                if(!base->CanApplyMacro(replacements, reason))
                    return false;
                if(!iter->CanApplyMacro(replacements, reason))
                    return false;
                return true;
            }

            MuRecursiveFunction* ApplyMacro(const map<string, MuRecursiveFunction*>&  replacements)
            {
                MuRecursiveFunction* temp = base->ApplyMacro(replacements);
                if(temp != base)
                {
                    delete base;
                    base = temp;
                }

                temp = iter->ApplyMacro(replacements);
                if(temp != iter)
                {
                    delete iter;
                    iter = temp;
                }
                return this;
            }       

            bool ContainsVariables() const
            {
                return base->ContainsVariables()
                    || iter->ContainsVariables();
            }
            
            bool CanComputeArity(string* reason) const
            {
                if(!base->CanComputeArity(reason))
                    return false;
                if(!iter->CanComputeArity(reason))
                    return false;
                if(iter->ComputeArity() != base->ComputeArity() + 2)
                {
                    *reason = "PrimitiveRecursion: arity of iterated function must be equal to arity of base function + 2";
                    return false;
                }
                return true;
            }
            
            size_t ComputeArity() const
            {
                return base->ComputeArity() + 1;
            }
            
            unsigned int Apply(const vector<unsigned int>& Parameters)
            {
                vector<unsigned int> copy(Parameters);
                unsigned int iterations = copy[copy.size()-1];
                copy.pop_back();
                
                bool RequiresIntermediateResults = iter->DependsOnParameter(copy.size()+1);
                bool CanGetStationary = !iter->DependsOnParameter(copy.size());
                unsigned int temp = 0;

                
                if(RequiresIntermediateResults || iterations == 0)
                    temp = base->Apply(copy);
                if(iterations == 0)
                    return temp;
                copy.push_back(0);
                copy.push_back(temp);                

                // optimization:
                //    if the "iter" function supports immediate loops
                if(iter->CanIterate())
                    return iter->ApplyIterated(iterations, copy);

                // if the end-result does not depend on the intermediate
                // results, then skip the iterations
                unsigned int i = RequiresIntermediateResults ? 1 : iterations;
                unsigned int nexttemp = 0;
                for(; i <= iterations; i++)
                {
                    copy[copy.size()-2] = i-1;
                    copy[copy.size()-1] = temp;
                    nexttemp = iter->Apply(copy);
                    if(CanGetStationary && (nexttemp == temp))
                        break;
                    temp = nexttemp;
                }
                return temp;
            }

            bool DependsOnParameter(size_t paramPosition) const
            {
                return base->DependsOnParameter(paramPosition)
                    || iter->DependsOnParameter(paramPosition)
                    || paramPosition == base->ComputeArity(); // number of iterations
            }
    };
    
    
    class MuRecursion : public MuRecursiveFunction
    {
        protected:
            MuRecursiveFunction* murec;
        public:
            MuRecursion(MuRecursiveFunction* murec)
                : MuRecursiveFunction(murec->arity > 0 ? murec->arity - 1 : 0)
            {
                this->murec = murec;
            }
            
            ~MuRecursion()
            {
                delete murec;
            }

            bool ValidStructure() const
            {
                return murec != NULL && murec->ValidStructure();
            }
            
            bool AllVariablesDefined(const vector<string>& names) const
            {
                return murec->AllVariablesDefined(names);
            }
            
            MuRecursiveFunction* Clone() const
            {
                return new MuRecursion(murec->Clone());
            }

            bool CanApplyMacro(const map<string, MuRecursiveFunction*>&  replacements, string* reason) const
            {
                return murec->CanApplyMacro(replacements, reason);
            }                   
            
            MuRecursiveFunction* ApplyMacro(const map<string, MuRecursiveFunction*>&  replacements)
            {
                MuRecursiveFunction* temp = murec->ApplyMacro(replacements);
                if(temp != murec)
                {
                    delete murec;
                    murec = temp;
                }
                return this;
            }                   

            bool ContainsVariables() const
            {
                return murec->ContainsVariables();
            }
            
            bool CanComputeArity(string* reason) const
            {
                if(!murec->CanComputeArity(reason))
                    return false;
                if(murec->ComputeArity() == 0)
                {
                    *reason = "MuRecursion: function must have arity at least 1";
                    return false;
                }
                return true;
            }
            
            size_t ComputeArity() const
            {
                return murec->ComputeArity() - 1;
            }
            
            unsigned int Apply(const vector<unsigned int>& Parameters)
            {
                if(murec->ComputeArity() == 0)
                    throw ;
            
                vector<unsigned int> copy = Parameters;
                copy.push_back(0);
                for(unsigned int i = 0; ; i++)
                {
                    copy[copy.size()-1] = i;
                    if(murec->Apply(copy) == 0)
                        return i;
                }
            }
            
            bool DependsOnParameter(size_t paramPosition) const
            {
                return murec->DependsOnParameter(paramPosition);
            }            
    };
    
    
    void yyerror(const char* s)
    {
        cerr << "SYNTACTIC ERROR (Line " << yylineno << "): " << s << endl;
    }
%}

    //--------------------------------------------------------------------------

%start commands
%union {
    string* text;
    unsigned int* number;
    MuRecursiveFunction* function;
    vector<unsigned int>* ints;
    vector<string>* ids;
    vector<MuRecursiveFunction*>* functions;
}
%token <text> ID
%token <number> INT
%token EQ OPEN COMMA CLOSE OPENBRACKET CLOSEBRACKET SEMICOLON POW INDEX PROJ SUCC REC COMP MU ARITY

%%


commands
    : commands command
    | 
    ;
             
command
    : ID EQ function SEMICOLON
    {
        bool ok = true;
        if(macros.find(*$<text>1) == macros.end())
        {
            macros[*$<text>1] = NULL; // this is here so that invalid macros don't lead to
                                      // "undefined variable" messages in macros that use them
            
            if($<function>3 != NULL // macros are not required to have valid arity
            && $<function>3->ValidStructure())
            {
                if(!$<function>3->ContainsVariables())
                {
                    macros[*$<text>1] = new MuRecursiveFunctionMacro(new vector<string>(), $<function>3);
                }
                else
                {
                    cerr << "Macro \"" << *$<text>1 << "\" contains undefined variables\n\n";
                    ok = false;
                }
            }
            else
            {
                cerr << "Invalid macro \"" << *$<text>1 << "\"\n\n";
                ok = false;
            }
        }
        else
        {
            cerr << "Macro \"" << *$<text>1 << "\" already defined!" << "\n\n";
            ok = false;
        }
        if(!ok)
        {
            delete $<function>3;
        }
        delete $<text>1;
    }
        
    | ID OPENBRACKET ids CLOSEBRACKET EQ function SEMICOLON
    {
        bool ok = true;
        if(macros.find(*$<text>1) == macros.end())
        {
            macros[*$<text>1] = NULL; // this is here so that invalid macros don't lead to
                                      // "undefined variable" messages in macros that use them
        
            if($<function>6 != NULL // macros are not required to have valid arity
            && $<function>6->ValidStructure())
            {
                if($<function>6->AllVariablesDefined(*$<ids>3))
                    macros[*$<text>1] = new MuRecursiveFunctionMacro($<ids>3, $<function>6);
                else
                {
                    cerr << "Macro \"" << *$<text>1 << "\" contains undefined variables\n\n";
                    ok = false;
                }
            }
            else
            {
                cerr << "Invalid macro \"" << *$<text>1 << "\"\n\n";
                ok = false;
            }
        }
        else
        {
            cerr << "Function " << *$<text>1 << " already defined!" << "\n\n";
            ok = false;
        }
        if(!ok)
        {
            delete $<ids>3;
            delete $<function>6;        
        }
        delete $<text>1;
    }
    
    | function ints SEMICOLON
    {
        MuRecursiveFunction* f = $<function>1;
        
        try
        {
            string reason;
            if(f != NULL)
            {
                if(f->CanApply(*$<ints>2, &reason))
                    cout << f->Apply( *$<ints>2 ) << "\n\n";
                else
                    cerr << "Cannot compute result, reason: " << reason << "\n\n";
            }
        }
        catch(const char* ex)
        {
            cerr << "Exception Caught: " << ex << "\n\n";
        }
        
        delete $<ints>2;
        delete f;
    }
    
    | function SEMICOLON
    {
        MuRecursiveFunction* f = $<function>1;
        
        try
        {
            string reason;
            vector<unsigned int> pseudoparams;
            if(f != NULL)
            {
                if(f->CanApply(pseudoparams, &reason))
                    cout << f->Apply( pseudoparams ) << "\n\n";
                else
                    cerr << "Cannot compute result, reason: " << reason << "\n\n";
            }
        }
        catch(const char* ex)
        {
            cerr << "Exception Caught: " << ex << "\n\n";
        }
        
        delete f;
    }
    
    | ARITY function SEMICOLON
    {
        MuRecursiveFunction* f = $<function>2;
        try
        {
            string reason;
            if(f != NULL
            && f->ValidStructure())
            {
                if(f->CanComputeArity(&reason))
                    cout << (f->ComputeArity()) << "\n\n";
                else
                    cerr << "Cannot compute arity: " << reason << "\n\n";
            }
        }
        catch(const char* ex)
        {
            cerr << "Exception Caught: " << ex << "\n\n";
        }
        
        delete f;
    }
    ;

function
    : ID
    {
        if(macros.find(*$<text>1) != macros.end())
        {
            try
            {
                MuRecursiveFunctionMacro* macro = macros[*$<text>1];
                if(macro != NULL)
                {
                    string reason;
                    vector<MuRecursiveFunction*> pseudoparams;
                    if(macro->CanInstantiate(&pseudoparams, &reason))
                        $<function>$ = macro->Instantiate(&pseudoparams);
                    else
                    {
                        cerr << "Cannot instantiate macro \"" << *$<text>1 << "\", reason: " << reason << "\n\n";
                        $<function>$ = NULL;
                    }
                }
                else
                {
                    cerr << "trying to instantiate invalid macro \"" << *$<text>1 << "\"\n\n";
                    $<function>$ = NULL;
                }
            }
            catch(const char* ex)
            {
                cerr << "Exception Caught: " << ex << "\n\n";
                $<function>$ = NULL;
            }
        }
        else
        {
            $<function>$ = new Variable(*$<text>1);
        }
        delete $<text>1;
    }

    | ID OPEN functions CLOSE
    {
        if(macros.find(*$<text>1) != macros.end())
        {
            try
            {
                MuRecursiveFunctionMacro* macro = macros[*$<text>1];
                if(macro != NULL)
                {
                    string reason;
                    if(macro->CanInstantiate($<functions>3, &reason))
                        $<function>$ = macro->Instantiate($<functions>3);
                    else
                    {
                        cerr << "Cannot instantiate macro \"" << *$<text>1 << "\", reason: " << reason << "\n\n";
                        $<function>$ = NULL;
                    }
                }
                else
                {
                    cerr << "trying to instantiate invalid macro \"" << *$<text>1 << "\"\n\n";
                    $<function>$ = NULL;
                }
            }
            catch(const char* ex)
            {
                cerr << "Exception Caught: " << ex << "\n\n";
                $<function>$ = NULL;
            }
        }
        else
        {
            cerr << "Error: undefined macro " << *$<text>1 << "\n\n";
            $<function>$ = NULL;
        }

        for(vector<MuRecursiveFunction*>::iterator i = $<functions>3->begin(); i != $<functions>3->end(); i++)
            delete *i;
        delete $<functions>3;
        delete $<text>1;
    }
    
    | INT POW INT
    {
        $<function>$ = new Constant(*$<number>3, *$<number>1);
        delete $<number>1;
        delete $<number>3;
    }

    | SUCC
    {
        $<function>$ = new Successor();
    }
    
    | PROJ INDEX INT POW INT
    {
        $<function>$ = new Projection(*$<number>5, *$<number>3 -1);
        delete $<number>5;
        delete $<number>3;
    }
    
    | REC OPEN function COMMA function CLOSE
    {
        $<function>$ = new PrimitiveRecursion($<function>3, $<function>5);
    }
    
    | COMP OPEN function COMMA functions CLOSE
    {
        $<function>$ = new Composition($<function>3, $<functions>5);
    }
   
    | MU function
    {
        $<function>$ = new MuRecursion($<function>2);
    }
    ;

functions
    : function
    {
        $<functions>$ = new vector<MuRecursiveFunction*>;
        $<functions>$->push_back($<function>1);
    }
    
    | functions COMMA function
    {
        $<functions>$ = $<functions>1;
        $<functions>$->push_back($<function>3);
    }
    ;
    
ints
    : INT
    {
        $<ints>$ = new vector<unsigned int>;
        $<ints>$->push_back(*$<number>1);
        delete $<number>1;
    }
    
    | ints INT
    {
        $<ints>$ = $<ints>1;
        $<ints>$->push_back(*$<number>2);
        delete $<number>2;
    }
    ;
    
ids
    : ID
    {
        $<ids>$ = new vector<string>;
        $<ids>$->push_back(*$<text>1);
        delete $<text>1;
    }
    
    | ids COMMA ID
    {
        $<ids>$ = $<ids>1;
        $<ids>$->push_back(*$<text>3);
        delete $<text>3;
    }
    ;    
%%

#ifdef TARGET_PLATFORM_NATIVE
int main(int argc, char** argv)
{
    int i;
    for(i=1; i < argc; i++)
    {
        string argvi = argv[i];
        if(argvi == "-v" || argvi == "--verbose")
            verbose = true;
        else if(argvi == "-s" || argvi == "--silent")
            silent = true;
        else if(argvi == "-dl")
            DEBUG_LEXER = true;
        else if(argvi == "-dp")
            DEBUG_PARSER = true;
        else if(argvi == "-h" || argvi == "--help")
        {
            cout << "\n";
            cout << "     \xce\xbc-recursive function interpreter\n";
            cout << "     ================================\n";
            cout << " \xc2\xa9 2015 by Viktor \"AlgorithMan\" Engelmann\n";
            cout << "               www.AlgorithMan.de\n";
            cout << " Distributed under the terms of the GNU general\n";
            cout << " public license (GPL) 3. See gpl3.txt for details\n";
            cout << "\n";
            cout << " usage: " << argv[0] << " [options] [sourcecodes]\n";
            cout << " where options can be\n";
            cout << "      -v --verbose\n";
            cout << "         prints intermediate steps\n";
            cout << "      -s --silent\n";
            cout << "         suppress messages\n";
            cout << "      -h --help\n";
            cout << "         this help message\n";
            cout << " example sourcecode:\n";
            cout << "      add = rec (   proj_1^1,   comp(succ, proj_3^3)   );\n";
            cout << "      mul = rec(   0^1,   comp(add, proj_1^3, proj_3^3)   );\n";
            cout << " example program call\n";
            cout << "      "<<argv[0]<<" stdfunctions.plc\n";
            cout << " example query\n";
            cout << "      add 4 5;\n";
            cout << " if you don\'t understand this whole \xce\xbc-recursion stuff, read\n";
            cout << "      https://en.wikipedia.org/wiki/%CE%9C-recursive_function\n";
            cout << "      https://en.wikipedia.org/wiki/Primitive_recursive_function\n";
            cout << "\n";
            return 0;
        }
        else
            break;
    }

    if(!silent)
    {
        cerr << "\n";
        cerr << "     \xce\xbc-recursive function interpreter\n";
        cerr << "     ================================\n";
        cerr << " \xc2\xa9 2015 by Viktor \"AlgorithMan\" Engelmann\n";
        cerr << "               www.AlgorithMan.de\n";
        cerr << " Distributed under the terms of the GNU general\n";
        cerr << " public license (GPL) 3. See gpl3.txt for details\n";
        cerr << "\n";
    }


    try
    {
        for(; i < argc; i++)
        {
            if(!silent) cerr << "loading file \"" << argv[i] << "\"...";
            yyin = fopen(argv[i], "r");
            yyparse();
            fclose(yyin);
            if(!silent) cerr << " done\n";
        }
        if(!silent) cerr << "\n";
        cerr.flush();


        yyin = stdin;
        yyparse();
    }
    catch(const char* ex)
    {
        cerr << argv[0] << " Fatal Exception: " << ex << "\n\n";
    }
        

    for(map<string,MuRecursiveFunctionMacro*>::iterator it = macros.begin(); it != macros.end(); it++)
        delete it->second;
    return 0;
}
#endif // TARGET_PLATFORM_NATIVE

#ifdef TARGET_PLATFORM_WASM
extern "C" double RunMuRecursion(char* code, double flags)
{
    int iflags = (int)flags;
    DEBUG_LEXER = ((iflags&1) != 0);
    DEBUG_PARSER = ((iflags&2) != 0);

    try
    {
        YY_BUFFER_STATE bufferstate = yy_scan_string(code);
        yyparse();
        yy_delete_buffer(bufferstate);
    }
    catch(const char* ex)
    {
        cerr << " Fatal Exception: " << ex << "\n\n";
    }
        
    for(map<string,MuRecursiveFunctionMacro*>::iterator it = macros.begin(); it != macros.end(); it++)
        delete it->second;
    macros.clear();
    
    return 0;
}
#endif // TARGET_PLATFORM_WASM
